#!/usr/bin/perl

use strict;

use CGI qw/:all *table/;
use DBI;
use Time::ParseDate;
# Note: more "USE" statements in &setup below to print fancy charts.

my ($sth, $dbh, $message, $rfc931_bg, $domain_bg, $starttime_bg, $endtime_bg,
$groups_bg, $remote_host_bg, $display_bg, $display, $alert_col, $output,
@rfc931_selected, @rfc931, @output, %output, %multquery, $rfc931, %acc_totals,
$acc_max, $max_factor, %time_options, %conf, @groups, $groups,
@groups_selected);

&setup;

if ($Q::submit eq 'Submit') {
	$message = check_valid();
	unless ($message) {
		my $query;
		if (($Q::display eq 'graphbytes') or ($Q::display eq 'graphpages')) {
			%multquery = make_query_graph();
			($message, $output) = search_db_graph();
		} else {
			$query = make_query();
			($message, $output) = search_db($query);
		}
	}
} elsif ($Q::submit eq 'Start again') {
	print header, qq|<META HTTP-EQUIV="refresh" CONTENT=0;URL="./squidalyser.pl">|;
	exit;
} else {
	$message = 'Squidalyser: interactive squid logfile analysis';
}

my $page = make_input_page($message, $output);
print $page;
dbdisconnect();
exit 0;

sub setup {
	$alert_col = '98cbcb';
	import_names('Q');
	@rfc931 = param('rfc931');
	@output = qw(listpages listsites pictures blocked graphbytes graphpages);
	%output = (
		listpages	=>	' A list of pages visited',
		listsites	=>	' A list of sites visited',
		pictures	=>      ' All the pictures viewed/downloaded',
		blocked		=>      ' Blocked or failed accesses',
		graphbytes	=>	' A graph of activity (per byte)',
		graphpages	=>	' A graph of activity (per item accessed)',
	);
	open (CONF, "< /usr/local/squidparse/squidalyser.conf") or die $!;
	while (my $line = <CONF>) {
		chomp($line);
		$line =~ /^\#/ and next;
		$line =~ /^$/ and next;
		my ($varname, $varvalue) = split(/\s+/, $line);
		$conf{$varname} = $varvalue;
	}
	close CONF;
	dbconnect($conf{dbhost}, $conf{dbname}, $conf{dbuser}, $conf{dbpass});
	# Bugfix on next line, from Julian S.
	if ($conf{timeformat} eq 'UK') {
		%time_options = ( UK => 1 );
	} else {
		%time_options = ( UK => 0 );
	}
	if ($conf{chartformat} eq 'fancy') {
		use GD::Graph::bars;
		use GD::Graph::colour;
		use GD::Graph::Data;
		use URI::Escape;
	}
	$groups = &get_group_names;
}

sub make_query_graph {
	my ($select_part, $rfc931_part, $domain_part, $starttime_part,
	$endtime_part, $remotehost_part, $where, %multquery);
	$rfc931 = \@rfc931;
	if ($rfc931[0] eq 'All') {
		if ($#rfc931 > 0) {
			shift(@{$rfc931});
		} else {
			$rfc931 = get_user_names();
			shift(@{$rfc931});
		}
	}
	foreach my $user (@{$rfc931}) {
		$user eq '-' and next;
		my $query;
		my ($domain_part, $where);
		my $user_q = $dbh->quote($user);
		if ($Q::display eq 'graphbytes') {
			$select_part = qq|
				SELECT SUM(bytes)
				FROM logfile
				WHERE rfc931 = $user_q
			|;
			$where++;
		} elsif ($Q::display eq 'graphpages') {
			$select_part = qq|
				SELECT COUNT(request)
				FROM logfile
				WHERE rfc931 = $user_q
			|;
			$where++;
		}
		if ($Q::domain) {
			($domain_part, $where) = domain_part($Q::domain, $where);
		} elsif ($Q::check_words) {
			($domain_part, $where) = word_check($where);
		}
		$Q::starttime and my ($starttime_part, $where) =
			starttime_part($Q::starttime, $where);
		$Q::endtime and my ($endtime_part, $where) =
			endtime_part($Q::endtime, $where);
		$Q::remotehost and my ($remotehost_part, $where) =
			remotehost_part($Q::remotehost, $where);
		my $query = qq|
			$select_part
			$rfc931_part
			$domain_part
			$starttime_part
			$endtime_part
			$remotehost_part
		|;
		$multquery{$user} = $query;
	}
	return %multquery;
}

sub make_query {
	my ($select_part, $rfc931_part, $domain_part, $starttime_part,
	$endtime_part, $remotehost_part, $where);

	if (($rfc931[0] eq 'All') and ($#rfc931 > 0)) { shift(@rfc931) };
	if ($Q::display eq 'listsites') {
		$select_part = 'SELECT id, rfc931, request FROM logfile';
	} elsif ($Q::display eq 'blocked') {
		$select_part = qq|
			SELECT * FROM logfile
			WHERE (status LIKE '40%' OR status LIKE '50%')
		|;
		$where++;
	} else {
		$select_part = 'SELECT * FROM logfile';
	}
	unless ($rfc931[0] eq 'All') {
		my @rfc931_temp = @rfc931;
		while ($#rfc931_temp > -1) {
			my $name = shift(@rfc931_temp);
			$name eq '-' and next;
			my $name_q = $dbh->quote($name);
			$rfc931_part .= qq|\nOR rfc931=$name_q|;
		}
		$rfc931_part .= ' ) ';
		$rfc931_part =~ s/OR/WHERE (/;
		$where and $rfc931_part =~ s/WHERE/AND /;
		$where++;
	}
	my $domain_part;
	if ($Q::domain) {
		($domain_part, $where) = domain_part($Q::domain, $where);
	} elsif ($Q::check_words) {
		($domain_part, $where) = word_check($where);
	}
	$Q::starttime and ($starttime_part, $where) =
		starttime_part($Q::starttime, $where);
	$Q::endtime and ($endtime_part, $where) =
		endtime_part($Q::endtime, $where);
	$Q::remotehost and ($remotehost_part, $where) =
		remotehost_part($Q::remotehost, $where);
	my $query = qq|
		$select_part
		$rfc931_part
		$domain_part
		$starttime_part
		$endtime_part
		$remotehost_part
		ORDER BY rfc931
	|;
	return $query;
}

sub domain_part {
	my ($domain, $where) = @_;
	# FIX: needs much refinement
	my $domain_q = $dbh->quote($domain);
	my $domain_part = qq|WHERE request LIKE \%$domain_q\%|;
	$domain_part =~ s/\'\%/\%\'/;
	$domain_part =~ s/\%\'/\'\%/;
	$where and $domain_part =~ s/WHERE/AND/;
	$where++;
	return $domain_part, $where;
}

sub word_check {
	my $where = shift;
	my $domain_part;
	my @words = &getwords;
	unless ($#words > -1) { return (undef, $where) };
	foreach my $word (@words) {
		my $word_q = $dbh->quote($word);
		$word_q = "\%$word_q\%";
		$word_q =~ s/\'\%/\%\'/;
		$word_q =~ s/\%\'/\'\%/;
		$domain_part .= qq| OR request LIKE $word_q|;
	}
	if ($where) {
		$domain_part =~ s/OR/AND (/;
		$domain_part .= ')';
	} else {
		$domain_part =~ s/OR/WHERE/;
		$where++;
	}
	return $domain_part, $where;
}

sub getwords {
	my @words;
	my $sth = $dbh->prepare('SELECT word FROM words_list ORDER BY word');
	$sth->execute or die $dbh->errstr;
	while (my $word = $sth->fetchrow) { push @words, $word };
	$sth->finish;
	return @words;
}

sub starttime_part {
	my ($starttime, $where) = @_;
	$starttime = parsedate($starttime, %time_options);
	my $starttime_part = qq|WHERE time >= $starttime|;
	$where and $starttime_part =~ s/WHERE/AND/;
	$where++;
	return $starttime_part, $where;
}

sub endtime_part {
	my ($endtime, $where) = @_;
	$endtime = parsedate($endtime, %time_options);
	my $endtime_part = qq|WHERE time <= $endtime|;
	$where and $endtime_part =~ s/WHERE/AND/;
	$where++;
	return $endtime_part, $where;
}

sub remotehost_part {
	my ($remotehost, $where) = @_;
	my $remotehost_q = $dbh->quote($Q::remotehost);
	my $remotehost_part = qq|WHERE remotehost=$remotehost_q|;
	$where and $remotehost_part =~ s/WHERE/AND/;
	$where++;
	return $remotehost_part, $where;
}

sub search_db_graph {
	my $message;
	my $output;
	if ($conf{chartformat} eq 'plain') {
		$output = &graph_plain;
	} else {
		$output = &graph_fancy;
	}
	$message = 'Finished';
	return $message, $output;
}

sub graph_fancy {
	my $output;
	if ($Q::display eq 'graphbytes') {
		$output .= 'title=Bytes%20downloaded&ylabel=Bytes&';
	} else {
		$output .= 'title=Items%20downloaded&ylabel=Items&';
	}
	foreach my $user (sort keys %multquery) {
		my $query = $multquery{$user};
		my $sth = $dbh->prepare($query);
		$sth->execute or die $dbh->errstr;
		my $total = $sth->fetchrow;
		$sth->finish;
		$total ||= 0;
		$user = uri_escape($user);
		$output .= qq|$user=$total&|;
	}
	$output =~ s/\&$//;
	return $output;
}

sub graph_plain {
	my $output;
	my $num_sq;
	if ($Q::display eq 'graphbytes') {
		$output = qq|<tr><th>User</th><th>Bytes</th><th>&nbsp;</th></tr>|;
	} elsif ($Q::display eq 'graphpages') {
		$output = qq|<tr><th>User</th><th>Items</th><th>&nbsp;</th></tr>|;
	}
	foreach my $user (sort keys %multquery) {
		my $query = $multquery{$user};
		my $sth = $dbh->prepare($query);
		$sth->execute or die $dbh->errstr;
		my $total = $sth->fetchrow;
		$acc_totals{$user} = $total;
		$acc_max += $total;
		$sth->finish;
	}
	$max_factor = int($acc_max / 100);
	$max_factor ||= 1;
	foreach my $user (sort keys %acc_totals) {
		$num_sq = $acc_totals{$user} / $max_factor;
		my $img = qq|<img src="../icons/dot100k.gif">| x $num_sq;
		my $total = commify($acc_totals{$user});
		$output .= qq|<tr><td>$user</td><td align="right">$total</td><td>$img</td></tr>|;
	}
	return $output;
}

sub search_db {
	my $query = shift;
	my $output;
	my %siteseen;
	if (($Q::display eq 'listpages') or ($Q::display eq 'blocked')) {
		$output = qq|<tr><td colspan=6><tt>$query</tt></td></tr>
		<tr><th>User</th><th>Time</th><th>Request</th><th>Status</th>
		<th>Bytes</th><th>Remote host</th></tr>|;
	} elsif ($Q::display eq 'listsites') {
		$output = qq|<tr><td colspan=2><tt>$query</tt></td></tr>
		<tr><th>User</th><th>Request</th></tr>|;
	} elsif ($Q::display eq 'pictures') {
		$output = qq|<tr><td colspan=6><tt>$query</tt></td></tr>
		<tr><th>User</th><th>Time</th><th>Image</th><th>Site</th>
		<th>Bytes</th><th>Remote host</th></tr>|;
	}
	my $date = localtime($Q::time);
	$date =~ s/\s+/\&nbsp\;/g;
	my $sth = $dbh->prepare($query);
	$sth->execute or die $dbh->errstr;
	if (($Q::display eq 'listpages') or ($Q::display eq 'blocked')){
		while (my($id, $remotehost, $rfc931, $authuser, $request,
		$status, $bytes, $time) = $sth->fetchrow) {
			my $date = localtime($time);
			$request = qq|<a href="$request">$request</a>|;
			$output .= qq|
				<tr><td><small>$rfc931</small></td>
				<td><small>$date</small></td>
				<td><small>$request</small></td>
				<td><small>$status</small></td>
				<td><small>$bytes</small></td>
				<td><small>$remotehost</small></td></tr>
			|;
		}
	} elsif ($Q::display eq 'listsites') {
		while (my ($id, $rfc931, $request) = $sth->fetchrow) {
			my $reference = qq|./squidalyser.pl?rfc931=$rfc931&domain=$Q::domain&starttime=$Q::starttime&endtime=$Q::endtime&remotehost=$Q::remotehost&display=listpages&submit=Submit|;
			$request = site_only($request);
			my $site = $rfc931 . site_name($request);
			$siteseen{$site}++ and next;
			$output .= qq|
				<tr><td><small><a href="$reference">$rfc931</a></small></td>
				<td><small>$request</small></td></tr>
			|;
		}
	} elsif ($Q::display eq 'pictures') {
		while (my($id, $remotehost, $rfc931, $authuser, $request,
		$status, $bytes, $time) = $sth->fetchrow) {
			if (is_graphic($request)) {
				my $date = localtime($time);
				my $url = url_only($request);
				$request = qq|<a href="$request">$request</a>|;
				$output .= qq|
					<tr><td><small>$rfc931</small></td>
					<td><small>$date</small></td>
					<td><a href="$url"><img src="$url" border=0></a></td>
					<td><small>$request</small></td>
					<td><small>$bytes</small></td>
					<td><small>$remotehost</small></td></tr>
				|;
			}
		}
	}
	$message = 'Finished';
	return $message, $output;
}

sub url_only {
	my $request = shift;
	if ($request =~ /^(GET\s+)(http:.*?)(\s+HTTP.*)$/) { $request = $2 };
	return $request;
}

sub is_graphic {
	my $request = shift;
	if ($request =~ /(\.jpg|\.gif|\.JPG|\.GIF|\.png|\.PNG)/) { return 1 };
	return 0;
}

sub hilite_url {
	my $request = shift;
	$request =~ s/^(GET\s+)(http:.*?)(\s+HTTP.*)$/$1<a href="$2" target="_showme">$2<\/a>$3/g;
	return $request;
}

sub site_only {
	my $request = shift;
	$request =~ m|^(http:\/\/.*?/).*?$|g;
	$request = qq|<a href="$1" target="_showme">$1</a>|;
	return $request;
}

sub site_name {
	my $request = shift;
	if ($request =~ /(http:\/\/.*?\/)/) { $request = $1 };
	return $request;
}

sub check_valid {
	my $not_valid;
	my $message;
	if ($Q::groups ne 'No selected group') {
		my $group_q = $dbh->quote($Q::groups);
		my $query = qq|SELECT id FROM groups WHERE group_name=$group_q|;
		my $sth = $dbh->prepare($query);
		$sth->execute or die $dbh->errstr;
		my $group_id = $sth->fetchrow;
		$sth->finish;

		$query = qq|SELECT rfc931 FROM members WHERE group_id=$group_id|;
		$rfc931 = $dbh->selectcol_arrayref($query);
		@rfc931 = @{$rfc931};
	}
	if ($#rfc931 > -1) {
		foreach my $name (@rfc931) {
			unless ($name =~ /^[\w\.\-]+$/) {
				$message .= qq|<li>One of the usernames is
					not valid.<br>|;
				$rfc931_bg = $alert_col;
				last;
			}
		}
	} else {
		$message .= qq|<li>You must select a username or group which has some members.<br>|;
		$rfc931_bg = $alert_col;
	}
	if (length($Q::domain)) {
		if ($Q::domain =~ /^\d+\.\d+\.\d+\.\d+$/) {
			$not_valid = valid_ip($Q::domain);
		} elsif ($Q::domain =~ /^[\w\.\-]+$/) {
			$not_valid = valid_chars($Q::domain);
		}
		if ($not_valid) {
			$message .= qq|<li>The IP or domain you entered is
				not valid.<br>|;
			$domain_bg = $alert_col;
		}
	}
	if (length($Q::starttime)) {
		$not_valid = valid_time($Q::starttime);
		if ($not_valid) {
			$message .= qq|<li>The start time is not valid.<br>|;
			$starttime_bg = $alert_col;
		}
	}
	if (length($Q::endtime)) {
		$not_valid = valid_time($Q::endtime);
		if ($not_valid) {
			$message .= qq|<li>The end time is not valid.<br>|;
			$endtime_bg = $alert_col;
		}
	}
	if ($message) {
		$message = qq|<font color="darkred">The request failed,
			please try again:</font><p><ol>| .
		$message . qq|</ol></p>|;
		
	}
	return $message;
}

sub valid_time {
	# see if parsedate likes the time they entered
	my $date = shift;
	my $seconds = parsedate($date, %time_options);
	$seconds or return 1;
	return 0;
}

sub valid_chars {
	# return 0 if string passed contains A-Z, a-z, 0-9, hyphen, underscore or dot.
	my $string = shift;
	if ($string !~ /^[\w\.\-]+$/) {
		return 1;
	}
	return 0;
}

sub valid_ip {
	# check for xxx.xxx.xxx.xxx in passed string; return 0 if OK
	my $ip = shift;
	$ip =~ /^\d+\.\d+\.\d+\.\d+$/ or return 1;
	my @quads = split(/\./, $ip);
	foreach my $quad (@quads) {
		if (($quad > 254) or ($quad < 1) or (length($quad) > 3)) {
			return 1;
		}
	}
	return 0;
}

sub dbconnect {
	my ($dbhost, $dbname, $dbuser, $dbpass) = @_;
	$dbh = DBI->connect("DBI:mysql:$dbname:$dbhost",$dbuser,$dbpass)
		or die (DBI::errstr);
}

sub dbdisconnect {
	$sth and $sth->finish;
	($dbh and $dbh->disconnect) or die (DBI::errstr);
}

sub make_input_page {
	my ($message, $output) = @_;
	my $rfc931 = get_user_names();
	my $words_checked = param('check_words');
	if ($Q::submit) {
		@rfc931_selected = @{$rfc931};
	} else {
		@rfc931_selected = 'All';
	}
	my $form .=
	header . 
	start_html({-bgcolor=>'linen', -title=>'Squidalyser v0.2: Log Analysis'}).
	start_form.
	start_table(
		{-border=>0,
		-cellpadding=>10,
		-cellspacing=>0,
		-width=>'100%',
		-bgcolor=>'linen'}
	) .
	Tr({-valign=>'top'},
		td({-colspan=>2},
			a({-href=>'./squidalyser.pl'},
				img(
					{-src=>'/icons/sq-squid1.gif',
					-alt=>'squidalyser',
					-border=>0}
				)
			),
			a({-href=>'./wordlist.pl'},
				img(
					{-src=>'/icons/sq-wordl0.gif',
					-alt=>'word list',
					-border=>0}
				)
			),
			a({-href=>'./groupmanage.pl'},
				img(
					{-src=>'/icons/sq-group0.gif',
					-alt=>'group manager',
					-border=>0}
				)
			),
		)
	) .
	Tr({-valign=>'top'},
		td({-colspan=>2, -height=>2},
			img(
				{-src=>'/icons/blue-square.gif',
				-width=>'100%',
				-height=>4}
			)
		),
	) .
	Tr({-valign=>'top'},
		td({-colspan=>2, -bgcolor=>'linen'},
			b($message)
		),
	) .
	Tr({-valign=>'top'},
		td({-colspan=>2, -height=>2},
			img(
				{-src=>'/icons/blue-square.gif',
				-width=>'100%',
				-height=>4}
			)
		),
	) .
	Tr({-valign=>'top', -bgcolor=>"$groups_bg"},
		td(
			popup_menu(
				-name=>'groups',
				-values=>\@{$groups},
				-default=>\@groups_selected,
			)
		),
		td({-width=>'55%'},
			b('Groups'),
			p,
			small('Select a group name from the list (see the
			<a href="groupmanage.pl">Group Manager</a>) or select
			users from the menu below. A selected group takes priority
			over selected users.')
		),
	) .
	Tr({-valign=>'top', -bgcolor=>"$rfc931_bg"},
		td(
			scrolling_list(
				-name=>'rfc931',
				-values=>\@{$rfc931},
				-default=>\@rfc931_selected,
				-size=>8,
				-multiple=>'true'
			)
		),
		td({-width=>'55%'},
			b('Username (RFC931)'),
			p,
			small('You can select more than one user from the list.
			<i>All</i> will not be included in the list if there are
			other selections. <i>All</i> is not recommended: your
			browser may crash.')
		),
	) .
	Tr({-valign=>'top'},
		td({-colspan=>2, -height=>2},
			img(
				{-src=>'/icons/blue-square.gif',
				-width=>'100%',
				-height=>4})
			),
	) .
	Tr({-valign=>'top', -bgcolor=>"$domain_bg"},
		td(
			textfield(
				-name=>'domain',
				-size=>32,
				-maxlength=>64,
				-value=>$Q::domain
			),
			br,
			checkbox(
				-name=>'check_words',
				-label=>' Check against word list',
				-value=>'on',
				-checked=>$words_checked
			),
		),
		td(
			b('Sub-string match'),
			p,
			small('eg <i>www.yahoo.com</i>, <i>.ac.uk</i>,
			<i>212.219.8.163</i>, <i>/games</i>, <i>.gif</i>, etc.',
			br,
			'If you select the word list option, clear the entries in the
			textfield, or the word list will not be used.')
		),
	) .
	Tr({-valign=>'top'},
		td({-colspan=>2, -height=>2},
			img(
				{-src=>'/icons/blue-square.gif',
				-width=>'100%',
				-height=>4}
			)
		),
	) .
	Tr({-valign=>'top', -bgcolor=>"$starttime_bg"},
		td(
			textfield(
				-name=>'starttime',
				-size=>32,
				-maxlength=>64,
				-value=>$Q::starttime
			)
		),
		td(
			b('Start time'),
			p,
			small('Free-form entry of the start time for which results
			should be returned (default <i>all</i>).')
		),
	) .
	Tr({-valign=>'top'},
		td({-colspan=>2, -height=>2},
			img(
				{-src=>'/icons/blue-square.gif',
				-width=>'100%',
				-height=>4}
			)
		),
	) .
	Tr({-valign=>'top', -bgcolor=>"$endtime_bg"},
		td(
			textfield(
				-name=>'endtime',
				-size=>32,
				-maxlength=>64,
				-value=>$Q::endtime
			)
		),
		td(
			b('End time'),
			p,
			small('Free-form entry of the end time (default <i>until now</i>).')
		),
	) .
	Tr({-valign=>'top'},
		td({-colspan=>2, -height=>2},
			img(
				{src=>'/icons/blue-square.gif',
				-width=>'100%',
				-height=>4}
			)
		),
	) .
	Tr({-valign=>'top', -bgcolor=>"$remote_host_bg"},
		td(
			textfield(
				-name=>'remotehost',
				-size=>32,
				-maxlength=>64,
				-value=>$Q::remotehost
			)
		),
		td(
			b('The user\'s system'),
			p,
			small('Name or IP address of the user\'s computer.')
		),
	) .
	Tr({-valign=>'top'},
		td({-colspan=>2, -height=>2},
			img(
				{src=>'/icons/blue-square.gif',
				-width=>'100%',
				-height=>4}
			)
		),
	) .
	Tr({-valign=>'top', -bgcolor=>"$display_bg"},
		td(
			radio_group(
				-name=>'display',
				-values=>\@output,
				-labels=>\%output,
				-default=>"$display",
				-linebreak=>'true'
			)
		),
		td(
			b('Output format'),
			p,
			small('Specify what the search should return.')
		),
	) .
	Tr({-valign=>'top'},
		td({-colspan=>2, -height=>2},
			img(
				{-src=>'/icons/blue-square.gif',
				-width=>'100%',
				-height=>4}
			)
		),
	) .
	Tr({-valign=>'top'},
		td(
			submit(
				-name=>'submit',
				-value=>'Submit'
			),
			'&nbsp;',
			submit(
				-name=>'submit',
				-value=>'Start again'
			),
			'&nbsp;',
			reset,
		),
		td(
			b('Action'),
			p,
			small('<i>Submit</i> to start search.<br><i>Start again</i>
			resets all form items to the original state.')
		),
	) .
	Tr({-valign=>'top'},
		td({-colspan=>2, -height=>2},
			img(
				{-src=>'/icons/blue-square.gif',
				-width=>'100%',
				-height=>4}
			)
		),
	) .
	Tr({-valign=>'top'},
		td({-colspan=>2, -height=>2},
			p({-align=>'center'},
				small(a({-href=>'http://ababa.org/'}, 'Squidalyser Homepage'))
			)
		),
	) .
	end_table . end_form;
	if ($Q::submit eq 'Submit' and (($Q::display eq 'graphbytes') or ($Q::display eq 'graphpages'))) {
	$form .= start_table({-border=>0, -cellpadding=>5, -align=>'center'});
		if ($conf{chartformat} eq 'plain') {
			$form .= $output;
		} else {
			$form .= Tr(
				td({-colspan=>2},
					img({-src=>"/cgi-bin/squidimg.pl?$output"}),
				)
			);
		}
	} else {
		$form .= start_table({-border=>1, -cellpadding=>5, -align=>'center'}) . $output;
	}
	$form .= end_table . end_html;
	return $form;
}

sub get_user_names {
	my $query = qq|SELECT DISTINCT(rfc931) FROM logfile WHERE rfc931 != '-' ORDER BY rfc931|;
	my $rfc931 = $dbh->selectcol_arrayref($query);
	splice @{$rfc931}, 0, 0, 'All';
	return $rfc931;
}

sub get_group_names {
	my $query = qq|SELECT DISTINCT(group_name) FROM groups ORDER BY group_name|;
	my $groups = $dbh->selectcol_arrayref($query);
	splice @{$groups}, 0, 0, 'No selected group';
	return $groups;
}

sub commify {
	# from the perl cookbook
	my $text = reverse $_[0];
	$text =~ s/(\d\d\d)(?=\d)(?!\d*\.)/$1,/g;
	return scalar reverse $text;
}
