#!/usr/bin/perl

use strict;
 
use CGI qw/:all *table/;
use DBI;

my (%conf, $dbh, $sth, @db_word_list, $alert_col, $message, $new_words_bg,
$word_entry_bg, $word_list_bg, @to_remove);

&setup;

if ($Q::submit eq 'Add') {
	$message = check_valid();
	unless ($message) {
		&add_words;
		$message = 'Words added to list';
	}
} elsif ($Q::submit eq 'Remove') {
	@to_remove = param('word_list');
	$message = check_select();
	unless ($message) {
		&remove_words;
		$message = 'Words removed from list';
	}
} else {
	$message = "Manage the 'suspect' word list";
}

my $content = &make_input_page;
print header, $content;
&dbdisconnect;

exit;

sub setup {
	import_names('Q');
	$alert_col = '98cbcb';
	open (CONF, "< /usr/local/squidparse/squidalyser.conf") or die $!;
	while (my $line = <CONF>) {
		chomp($line);
		$line =~ /^\#/ and next;
		$line =~ /^$/ and next;
		my ($varname, $varvalue) = split(/\s+/, $line);
		$conf{$varname} = $varvalue;
	}
	close CONF;
	dbconnect($conf{dbhost}, $conf{dbname}, $conf{dbuser}, $conf{dbpass});
	@db_word_list = &getwords;
}

sub getwords {
	my @words;
	my $sth = $dbh->prepare('SELECT word FROM words_list ORDER BY word');
	$sth->execute or die $dbh->errstr;
	while (my $word = $sth->fetchrow) { push @words, $word };
	$sth->finish;
	return @words;
}

sub add_words {
	my %done;
	$Q::new_words =~ s/\s+//g;
	my @new_words = split(/,/, $Q::new_words);
	foreach my $word (@db_word_list) { $done{$word}++ };
	foreach my $word (@new_words) {
		$done{$word}++ and next;
		my $word_q = $dbh->quote($word);
		$dbh->do("INSERT INTO words_list VALUES (NULL, $word_q)") or die $dbh->errstr;
		push @db_word_list, $word;
	}
	@db_word_list = sort(@db_word_list);
}

sub remove_words {
	foreach my $word (@to_remove) {
		my $word_q = $dbh->quote($word);
		$dbh->do("DELETE FROM words_list WHERE word=$word_q") or die $dbh->errstr;
		splice(@db_word_list);
	}
	@db_word_list = &getwords;
}

sub check_select {
	my $message;
	unless ($#to_remove > -1) {
		$message = 'Please select a word (or words) from the list';
		$word_list_bg = $alert_col;
	}
	return $message;
}

sub check_valid {
	my $message;
	unless (length($Q::new_words)) {
		$message = 'Please enter a word or list of words in the box';
		$word_entry_bg = $alert_col;
	}
	return $message;
}

sub make_input_page {
	my $form =
	start_html({-bgcolor=>'linen', -title=>'Squidalyser v0.2: Word List Manager'}).
	start_form.
	start_table({-bgcolor=>'linen', -border=>0, -cellpadding=>10, -width=>'100%'}) .
	Tr({-valign=>'top'},
		td({-colspan=>3},
			a({-href=>'./squidalyser.pl'},
				img(
					{-src=>'/icons/sq-squid0.gif',
					-alt=>'squidalyser',
					-border=>0}
				)
			),
			a({-href=>'./wordlist.pl'},
				img(
					{-src=>'/icons/sq-wordl1.gif',
					-alt=>'word list',
					-border=>0}
				)
			),
			a({-href=>'./groupmanage.pl'},
				img(
					{-src=>'/icons/sq-group0.gif',
					-alt=>'group manager',
					-border=>0}
				)
			),
		)
	) .
	Tr({-valign=>'top'},
		td({-colspan=>3, -height=>2},
			img(
				{-src=>'/icons/blue-square.gif',
				-width=>'100%',
				-height=>4}
			)
		),
	) .
	Tr({-valign=>'top'},
		td({-colspan=>3},
			b($message)
		),
	) .
	Tr({-valign=>'top'},
		td({-colspan=>3, -height=>2},
			img(
				{-src=>'/icons/blue-square.gif',
				-width=>'100%',
				-height=>4}
			)
		),
	) .
	Tr({-valign=>'top', -bgcolor=>"$word_entry_bg"},
		td({-width=>'40%'},
			textfield(
				-name=>'new_words',
				-size=>40,
				-maxlength=>64,
				-value=>$Q::new_words
			)
		),
		td({-width=>'10%'},
			submit(
				-name=>'submit',
				-value=>'Add'
			)
		),
		td(
			b('Add words to list'),
			p,
			small('One word or a comma-separated list.')
		),
	) .
	Tr({-valign=>'top'},
		td({-colspan=>3, -height=>2},
			img(
				{-src=>'/icons/blue-square.gif',
				-width=>'100%',
				-height=>4}
			)
		),
	) .
	Tr({-valign=>'top', -bgcolor=>"$word_list_bg"},
		td(
			scrolling_list(
				-name=>'word_list',
				-values=>\@db_word_list,
				-multiple=>'true',
				-size=>15
			)
		),
		td(
			submit(
				-name=>'submit',
				-value=>'Remove'
			)
		),
		td(
			b('Remove words from list'),
			p,
			small('Select a word or multiple words.')
		),
	) .
	Tr({-valign=>'top'},
		td({-colspan=>3, -height=>2},
			img(
				{-src=>'/icons/blue-square.gif',
				-width=>'100%',
				-height=>4}
			)
		),
	) .
	Tr({-valign=>'top'},
		td({-colspan=>3, -height=>2},
			p({-align=>'center'},
				small(a({-href=>'http://ababa.org/'}, 'Squidalyser Homepage'))
			)
		),
	) .
	end_table . end_form . end_html;
	return $form;
}

sub dbconnect {
	my ($dbhost, $dbname, $dbuser, $dbpass) = @_;
	$dbh = DBI->connect("DBI:mysql:$dbname:$dbhost",$dbuser,$dbpass) or die (DBI::errstr);
}

sub dbdisconnect {
	$sth and $sth->finish;
	($dbh and $dbh->disconnect) or die (DBI::errstr);
}
